<?php

/**
 * WC_Fortis_Checkout_Redirect Force user login/registration for save card functionality
 *
 * @category Class
 * @package  Woocommerce
 * @author   None <none@fake.com>
 * @license  http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * @link     http://www.none.com/
 */
class WC_Fortis_Checkout_Redirect {

const FILE = __FILE__;
const URL_ARG = 'redirect_to_checkout_purchase';

protected static $_instance = null;

protected function __construct () {
    add_action( 'plugins_loaded', [ $this, 'init' ] );
}

protected function is_woocommerce_installed () {
    return function_exists( 'WC' );
}

protected function has_query_param () {
    return isset( $_GET[ self::URL_ARG ] );
}

protected function get_login_page_url () {
    return apply_filters( 'wc_fortis_redirect_login_page_url',
        get_permalink( get_option( 'woocommerce_myaccount_page_id' ) )
    );
}

protected function get_checkout_page_url () {
    return apply_filters( 'wc_fortis_redirect_checkout_page_url', wc_get_checkout_url() );
}

public function init () {
    if ( ! $this->is_woocommerce_installed() ) {
        add_action( 'admin_notices', [ $this, 'add_admin_notice' ] );
        return;
    };

    $options = get_option("woocommerce_". GW_ZP_ID ."_settings");
    $saveCards = $options['save_cards'];

    if ($saveCards !== 'yes'){
        return;
    }

    add_action( 'template_redirect', [ $this, 'redirect_to_account_page' ] );
    add_action( 'wp_head', [ $this, 'add_wc_notice' ] );

    add_filter( 'woocommerce_registration_redirect', [ $this, 'redirect_to_checkout' ], 100 );
    add_filter( 'woocommerce_login_redirect', [ $this, 'redirect_to_checkout' ], 100 );
    add_action( 'wp_head', [ $this, 'redirect_to_checkout_via_html' ] );
}

public function redirect_to_account_page () {
    $condition = apply_filters(
        'wc_fortis_redirect_redirect_to_account_page',
        is_checkout() && ! is_user_logged_in()
    );
    if( $condition ) {
        wp_safe_redirect( add_query_arg( self::URL_ARG, '', $this->get_login_page_url() ) );
        die;
    }
}

public function redirect_to_checkout_via_html () {
    if ( $this->has_query_param() && is_user_logged_in() ) {
        ?>
        <meta
            http-equiv="Refresh"
            content="0; url='<?php echo esc_attr( $this->get_checkout_page_url() ); ?>'"
        />
        <?php
        exit();
    }
}

public function redirect_to_checkout ( $redirect ) {
    if ( $this->has_query_param() ) {
        $redirect = $this->get_checkout_page_url();
    }
    return $redirect;
}

public function get_alert_message () {
    return apply_filters( 'wc_fortis_redirect_message', __( 'Please log in or register to complete your purchase.', 'wc-fortis-redirect' ) );
}

public function add_wc_notice () {
    if ( ! is_user_logged_in() && is_account_page() && $this->has_query_param() ) {
        wc_add_notice( $this->get_alert_message(), 'notice' );
    }
}

public function add_admin_notice () {
    ?>
    <div class="notice notice-error">
        <p>
            <?php echo esc_html__( 'You need install and activate the WooCommerce plugin.', 'wc-fortis-redirect' ) ?>
        </p>
    </div>
    <?php
}

public static function get_instance() {
    if ( is_null( self::$_instance ) ) {
        self::$_instance = new self();
    }
    return self::$_instance;
}
}