<?php

/**
 * WC_Zeamster_Gateway_Card Main implementation of GW's card processing
 *
 * @category Class
 * @package  Woocommerce
 * @author   None <none@fake.com>
 * @license  http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * @link     http://www.none.com/
 */
class WC_Zeamster_Gateway_Card extends WC_Payment_Gateway_CC
{
	private $_logger;
	private $options;

    /**
     * Constructor method
     */
    public function __construct()
    {
		$this->id = GW_ZP_ID;
		$this->options = get_option("woocommerce_". GW_ZP_ID ."_settings");
		$this->_logger = new ZP_Logger();
		$this->_logger->setFebug(($this->options && $this->options["debug"] === "yes") ? true : false);
		$this->title = __( $this->options ? $this->options['title'] : '', GW_ZP_ID );
		$this->method_title = __( GW_METHOD_TITLE_CC ,  GW_ZP_ID);
        $this->icon = apply_filters(GW_ZP_ID.'_icon', '');
		$this->has_fields = true;

        $this->supports = array(
			'zeamster',
			'products',
            'default_credit_card_form',
            'refunds',
            'tokenization'
        );

        $this->card_type_options = apply_filters(
            GW_ZP_PLUGIN_ID.'_card_types',
            $this->_getCardTypeOptions()
		);
		
        // Load the form fields.
        $this->init_form_fields();
        $this->init_settings();

        // Actions
        $updateOpts = 'woocommerce_update_options_payment_gateways';
        add_action($updateOpts, array($this, 'process_admin_options'));
		add_action($updateOpts.'_'.$this->id, array($this, 'process_admin_options'));
		add_action( 'woocommerce_receipt_'.GW_ZP_ID, array( $this, 'receipt_page' ) );
	}
	
    /**
     * Initialize Gateway Settings Form Fields (override of the abstract method)
     *
     * @return void
     */
    public function init_form_fields()
    {
        $lfp = WC_Log_Handler_File::get_log_file_path(GW_ZP_ID);

        $this->form_fields = array(
          
			'enabled' => array(
				'title'   => __('Enable/Disable', GW_ZP_PLUGIN_ID),
				'type'    => 'checkbox',
				'label'   => __('Enable Payment', GW_ZP_PLUGIN_ID),
				'default' => 'yes'
			),
			
			'environment' => array(
				'title' => __('Environment', GW_ZP_ID),
				'type' => 'select',
				'class' => 'wc-enhanced-select',
				'options' => array(
					'production' => 'Production',
					'sandbox' => 'Developer (Test) Mode',
				
				),
				'description' => __('Select the transaction environment. Selecting Developer (Test) Mode will allow you to run Test Transactions.', GW_ZP_ID),
				'default' => 'production',
				'desc_tip' => true,
			),

			'title' => array(
				'title'       => __('Title', GW_ZP_ID),
				'type'        => 'text',
				'description' => __(
					'This controls the title for the payment 
					method the customer sees during checkout.',
					GW_ZP_ID
				),
				'default'     => __('Credit Card'),
				'desc_tip'    => true,
			),

			'production_host_domain' => array(
				'title' => __('API Host', GW_ZP_ID),
				'type' => 'text',
				'description' => __(
					'Please enter your API Host.',
					GW_ZP_ID
				),
				'default' => '',
				'class' => 'production_input',
				'desc_tip' => true,
			),

			'production_user_id' => array(
				'title' => __('User ID', GW_ZP_ID),
				'type' => 'text',
				'description' => __(
					'Please enter your User ID.',
					GW_ZP_ID
				),
				'default' => '',
				'class' => 'production_input',
				'desc_tip' => true,
			),

			'production_user_api_key' => array(
				'title' => __('User API Key', GW_ZP_ID),
				'type' => 'password',
				'description' => __(
					'Please enter your User API Key.',
					GW_ZP_ID
				),
				'default' => '',
				'class' => 'production_input',
				'desc_tip' => true,
			),

			'production_hpp_id' => array(
				'title' => __('HPP ID', GW_ZP_ID),
				'type' => 'text',
				'description' => __(
					'Please enter your Hosted Payment Page ID.',
					GW_ZP_ID
				),
				'default' => '',
				'class' => 'production_input',
				'desc_tip' => true,
			),

			'production_hpp_encryption_key' => array(
				'title' => __('HPP Encryption Key', GW_ZP_ID),
				'type' => 'password',
				'description' => __(
					'Please enter your Hosted Payment Page Encryption Key.',
					GW_ZP_ID
				),
				'default' => '',
				'class' => 'production_input',
				'desc_tip' => true,
			),

			'sandbox_host_domain' => array(
				'title' => __('Sandbox API Host', GW_ZP_ID),
				'type' => 'text',
				'description' => __(
					'Please enter your Sandbox API Host.',
					GW_ZP_ID
				),
				'default' => '',
				'class' => 'sandbox_input',
				'desc_tip' => true,
			),

			'sandbox_user_id' => array(
				'title' => __('Sandbox User ID', GW_ZP_ID),
				'type' => 'text',
				'description' => __(
					'Please enter your User ID.',
					GW_ZP_ID
				),
				'default' => '',
				'class' => 'sandbox_input',
				'desc_tip' => true,
			),

			'sandbox_user_api_key' => array(
				'title' => __('Sandbox User API Key', GW_ZP_ID),
				'type' => 'password',
				'description' => __(
					'Please enter your User API Key.',
					GW_ZP_ID
				),
				'default' => '',
				'class' => 'sandbox_input',
				'desc_tip' => true,
			),

			'sandbox_hpp_id' => array(
				'title' => __('Sandbox HPP ID', GW_ZP_ID),
				'type' => 'text',
				'description' => __(
					'Please enter your Hosted Payment Page ID.',
					GW_ZP_ID
				),
				'default' => '',
				'class' => 'sandbox_input',
				'desc_tip' => true,
			),

			'sandbox_hpp_encryption_key' => array(
				'title' => __('Sandbox HPP Encryption Key', GW_ZP_ID),
				'type' => 'password',
				'description' => __(
					'Please enter your Hosted Payment Page Encryption Key.',
					GW_ZP_ID
				),
				'default' => '',
				'class' => 'sandbox_input',
				'desc_tip' => true,
			),

			'transactiontype' => array(
				'title' => __('Transaction Type', GW_ZP_ID),
				'type' => 'select',
				'class' => 'wc-enhanced-select',
				'options' => array(
					'sale' => 'Sale (Authorize and Capture)',
					'authonly' => 'Authorize Only',
				),
				'description' => __('Select your Transaction Type.', GW_ZP_ID),
				'default' => 'sale',
				'desc_tip' => true,
			),

			'auto_decline_cvv' => array(
				'title'   => __('Auto Decline CVV', GW_ZP_ID),
				'type'    => 'checkbox',
				'label'   => __('Auto Decline CVV', GW_ZP_ID),
				'description' => __('If the CVV (numbers on back of card) entered does not match that registered to the card, the transaction will be automatically declined.', GW_ZP_ID),
				'default' => 'yes',
				'desc_tip' => true
			),

			'auto_decline_street' => array(
				'title'   => __('Auto Decline Street', GW_ZP_ID),
				'type'    => 'checkbox',
				'label'   => __('Auto Decline Street', GW_ZP_ID),
				'description' => __('If the street entered does not match what is registered to the card, the transaction will be automatically declined.', GW_ZP_ID),
				'default' => 'no',
				'desc_tip' => true
			),

			'auto_decline_zip' => array(
				'title'   => __('Auto Decline Zip', GW_ZP_ID),
				'type'    => 'checkbox',
				'label'   => __('Auto Decline Zip', GW_ZP_ID),
				'description' => __('If the Zip Code entered does not match what is registered to the card, the transaction will be automatically declined.', GW_ZP_ID),
				'default' => 'no',
				'desc_tip' => true
			),

			'save_cards' => array(
				'title' => __('Save Cards', GW_ZP_ID),
				'type' => 'checkbox',
				'description' => __(
					'If enabled, users will be able to pay with a saved card '.
					'during checkout. Card details are saved on Fortis Payments\' '.
					'servers, not on your store.<br/><br/>'.
					'This requires customers to login/register before checking out. '.
					'Be sure to go to WooCommerce Settings/Accounts & Privacy and select <br>'.
					'"Allow Allow customers to create an account on the "My account" page".',
					GW_ZP_ID
				),
				'default' => 'no',
				//'desc_tip' => true,
			),

			'debug' => array(
				'title' => __('Debug log', GW_ZP_ID),
				'type' => 'checkbox',
				'label' => __('Enable logging', GW_ZP_ID),
				'default' => 'no',
				'description' => sprintf(
					__('Log requests, inside %s', GW_ZP_ID), '<code>'.$lfp.'</code>'
				),
			),
		);
	}
	
	/**
	 * Admin Panel Options
	 * Include Zeamster logo and add some JS for revealing inputs for sandbox vs production
	 *
	 * @access public
	 * @return void
	 */
	public function admin_options() {
		?>
		
		<img style="margin:10px -6px" width="218" height="55" src="<?php echo plugins_url('assets/images/logo.png', dirname(__FILE__)) ?>"/>
		
		<?php //if (empty($this->api_credentials['mid'])): ?>
			<div class="zeamster-banner updated">
				<p class="main">
				<h3 style="margin:0;"><?php _e('Getting started', 'woocommerce'); ?></h3></p>
				<p>
					<?php _e('Fortis is a leading provider of payment processing and technology services that helps thousands of merchants across the U.S. accept billions of dollars in card transactions each year. ', 'woocommerce'); ?>
				</p>
				<p>
					<a href="https://www.fortispay.com/" target="_blank" class="button button-primary"><?php _e('Visit Fortis', 'woocommerce'); ?></a>
				</p>
			</div>
		<?php //endif; ?>

		<table class="form-table">
			<?php $this->generate_settings_html(); ?>

			<script type="text/javascript">
				jQuery('#woocommerce_<?php echo GW_ZP_ID; ?>_environment').on('change', function () {
					var sandbox = jQuery('.sandbox_input').closest('tr');
					var production = jQuery('.production_input').closest('tr');
					var environment = this.value;

					if (environment === "sandbox") {
						sandbox.show();
						production.hide();
					} else {
						sandbox.hide();
						production.show();
					}
				}).change();
				
				jQuery(function ($) {
					$('#mainform').submit(function (e) {
						if ($('#woocommerce_<?php echo GW_ZP_ID; ?>_environment').val() === "production") {
							var allowSubmit = true;
							$('.production_input').each(function () {
								if ($(this).val() == '') allowSubmit = false;
							});
							if (!allowSubmit) alert('Warning! In order to enter Live Production Mode you must enter values for both User ID, User API Key, HPP URL and HPP Encryption Key.');
							return allowSubmit;
						}
					});
				});
			</script>
		</table>
		<?php
	}

	/**
     * Payment section's fields (override of the abstract method)
     *
     * @return Void
     **/
    public function payment_fields()
    {
		$zpfn = new WC_Zeamster_Functions();
		$configCheck = $zpfn->checkZpConfig();
		$saveCards = $this->options[ 'save_cards' ];

		if ( !$configCheck ) {
			wc_clear_notices(); // Clear Duplicate Notice
			wc_add_notice( "Payments are currently unavailable. Contact Store Owner", 'error' );
		}
		
		$environment = $zpfn->getEnvironment();

		if ( $configCheck && $environment === "sandbox" ) {
			echo '<div style="text-align:center;">
					<strong>TEST MODE is currently enabled.</strong><br/>Transactions processed in Test Mode will not process real money.
					<hr/>
				  </div>';
		}

		if ( is_checkout()) {
			?>
			<div>
				Please click "<strong>Pay Securely</strong>" to proceed to the payment screen. 
					
			<?php 
				if ( is_user_logged_in() && $this->supports( 'tokenization' ) && $saveCards === "yes" ) {
					?>
						If you would like to save your payment 
						method for future purchases, please select "<strong>Save to Account</strong>" before proceeding.
						<br><br>
						<?php
							if ( count( $this->get_tokens() ) > 0 ) { 
								$this->saved_payment_methods();
							}	
							?>
								<p>&nbsp</p>
							<?php
								
								$this->save_payment_method_checkbox();
				}
			?>
			</div>
			<?php
			
		}
	}

    /**
     * Payment process action (override of the abstract method)
     *
     * @param Int $orderID ID of the order
     *
     * @return Array
     */
    public function process_payment( $order_id )
    {
		$zpfn = new WC_Zeamster_Functions();
		$configCheck = $zpfn->checkZpConfig();
		
		if ( !$configCheck ) {
			wc_clear_notices(); // Clear Duplicate Notice
			wc_add_notice( "Payments are currently unavailable. Contact Store Owner", 'error' );
		}
		
		//get woo version
		if ( class_exists( 'WooCommerce' ) ) {
            global $woocommerce;
            $woo_version = $woocommerce->version;
		} else { 
			$woo_version = "old";
		}
        
        $order = new WC_Order( $order_id );
        $order_key = $order->get_order_key();  
        
        //get redirect url based on the version
        if ( $woo_version == 'old' )  {
			$redirect = add_query_arg( 'order', $order->get_id(), add_query_arg( 'key', $order_key, get_permalink( get_option( 'woocommerce_pay_page_id' ) ) ) );
		} else { 
			$redirect = $order->get_checkout_payment_url( true );
		} 

		// Check if using a payment token
		if ( isset( $_POST[ 'wc-' . GW_ZP_ID . '-payment-token' ] ) ) { 
			if ( $_POST[ 'wc-' . GW_ZP_ID . '-payment-token' ] !== "new" ) {
				$token_id = $_POST['wc-'. GW_ZP_ID .'-payment-token'];
				$token = WC_Payment_Tokens::get( $token_id );
				$response = $zpfn->processSaleFromAv( $token, $order ); 				

				$this->_logger->info(json_encode($response));

				if ( isset( $response )) { 				
					if ( isset( $response["transaction"]) ) { 

						$this->_logger->info(json_encode($response));

						$redirect = $this->updateSuccessfulOrder( $order_id, $response["transaction"], false );
						
						return array(
							'result'   => 'success',
							'redirect' => $redirect 
						);
					}
				}
			}
		} 
		
		if ( !isset ( $_POST[ 'wc-' . GW_ZP_ID . '-payment-token' ] ) || $_POST[ 'wc-' . GW_ZP_ID . '-payment-token' ] === "new" ) {
			$saveCard = "false";

			if ( isset( $_POST[ 'wc-'. GW_ZP_ID .'-new-payment-method' ] ) ) {
				$saveCard = $_POST['wc-'. GW_ZP_ID .'-new-payment-method'];
			}

			return array(
				'result'   => 'success',
				'redirect' => $redirect . '&save_card=' . $saveCard,
			);
		}

        
	}

	public function receipt_page( $order_id ) {
		$order = new WC_Order( $order_id );
		$zpfn  = new WC_Zeamster_Functions();
		wp_enqueue_script('process_payment', plugin_dir_url(__FILE__) . '../javascript/process_payment.js');
	
		$configCheck = $zpfn->checkZpConfig();
		if ( !$configCheck ) {
			wc_clear_notices(); // Clear Duplicate Notice
			wc_add_notice( "Payments are currently unavailable. Contact Store Owner", 'error' );
		}

		$order = new WC_Order($order_id);
		$saveCard = $_GET[ 'save_card' ];
		$args = $zpfn->getHppRequestData( $order_id, $order, $saveCard );
		$data = $zpfn->encodeRequestData( $args );
		$url  = $zpfn->getHppUrl( $data );
		?>

		<style>
			#zeamster_iframe { 
				width: 100%; height: 600px; border: 0;
			}
		</style>
		<?php
			$allowedUrl = $zpfn->getAllowedPostbackUrl();

			if ($allowedUrl !== false) {			
				?>
				<script>
					window.addEventListener("message", receiveMessage, false);

					function receiveMessage(event) {
						// Make sure the value for allowed matches the domain of the iFrame you are embedding.
						var allowed = "<?php echo $allowedUrl; ?>";
						// Verify sender's identity
						if (event.origin !== allowed) return;

						var data = event.data;
						var parsedData = JSON.parse(data);
						const successResponseCodes = [101, 102, 301];
						const saveCard = <?php echo $saveCard; ?>;
                        
                        
						if ( !successResponseCodes.includes(parsedData.status_id) ) { 
						    alert('test payment');
						    console.log(parsedData);
							let cl = document.getElementById('zp_decline_message').classList;
							let src = "<?php echo $url ?>"
							let iframe = document.getElementById('<?php echo GW_ZP_ID .'_iframe' ?>')
							iframe.src = src
							cl.toggle('fade')
							return false;
						}

						receiveTransactionAction(parsedData)
					}
				</script>
				<!--<h2> Please complete the payment form below to confirm your order </h2> -->
				<input type="hidden" name="zp_order_number" id="zp_order_number" value="<?php echo $order_id; ?>">
				<input type="hidden" name="zp_txn_data" id="zp_txn_data" value="">
 				<div class="alert alert-danger alert-dismissible fade show" role="alert" id="zp_decline_message">
					<strong>Transaction has been declined. Please re-enter your information to retry.</strong>
					<button type="button" class="close" data-dismiss="alert" aria-label="Close">
						<span aria-hidden="true">&times;</span>
					</button>
				</div>
				<iframe src="<?php echo $url ?>" id="<?php echo GW_ZP_ID .'_iframe' ?>" scrolling="no"></iframe>
				<?php
			} else {
				?>
				<h2> Unable to load payment form. Contact store owner.</h2> 
				<?php
			}
	}

	public function updateSuccessfulOrder( $order_id, $zp_response, $hpp ) { 		
		global $woocommerce;

		$zpfn  = new WC_Zeamster_Functions();
		$configCheck = $zpfn->checkZpConfig();
		
		if ( !$configCheck ) {
			wc_clear_notices(); // Clear Duplicate Notice
			wc_add_notice( "Payments are currently unavailable. Contact Store Owner", 'error' );
		}

		$order = new WC_Order( $order_id );
		$redirecturl = $this->get_return_url( $order );
		
		if ( in_array( $zp_response[ 'status_id' ],  [ '101', '102', '301' ] ) && $order->get_status() != 'Completed' ) {
			if ($hpp === true) { 
				$transaction_id = $zp_response [ 'transaction_id' ];
			} else { 
				$transaction_id = $zp_response [ 'id' ];
			}


			// Payment has been successful
			$order->add_order_note( __( 'Gateway payment completed.', 'zeamster' ) );

			// Add notes to the order
			$note = "Order Details:\n";
			$note.= "Transaction ID: ". sanitize_text_field( $transaction_id )."\n";
			$note.= "Result Code Text: ". sanitize_text_field( $zp_response [ 'status_id' ] )."\n";
			$note.= "Authorization Code: ". sanitize_text_field( $zp_response [ 'auth_code' ] )."\n";
			$note.= "Address Match: ". sanitize_text_field( $zp_response [ 'avs' ] );
			$order->add_order_note( __( $note, 'zeamster' ) );

			// Add Status ID to DB
			update_post_meta( $order_id, '_payment_gateway_status', $zp_response['status_id'] );

			// Handle Authorize Only
			if ( $zp_response[ 'status_id' ] == '102' ) {
				// Store captured value
				$order->update_meta_data( '_zp_charge_captured', 'no' );
				$order->update_meta_data( '_zp_transaction_id', $transaction_id );

				// Mark as on-hold - We will use this status for Auth Only Transactions
				$authorized_message = sprintf( __( 'Gateway charge authorized (Charge ID: %s). Process order by changing the status to "processing" or "completed" to take payment, or "cancelled" to remove the pre-authorization.', GW_ZP_ID ), $transaction_id );
				$order->set_transaction_id( $transaction_id );
				$order->update_status( 'on-hold', $authorized_message );
			}
			
			// Handle Authorize Only
			if ( $zp_response[ 'status_id' ] == '301' ) {
			    //echo 'test failed payment';exit;
				// Store captured value
				$order->update_meta_data( '_zp_charge_captured', 'no' );
				$order->update_meta_data( '_zp_transaction_id', $transaction_id );

				// Mark as on-hold - We will use this status for Auth Only Transactions
				$authorized_message = sprintf( __( 'Gateway charge authorized (Charge ID: %s). Process order by changing the status to "processing" or "completed" to take payment, or "cancelled" to remove the pre-authorization.', GW_ZP_ID ), $transaction_id );
				$order->set_transaction_id( $transaction_id );
				$order->update_status( 'failed', $authorized_message );
			}

			// Handle Auth/Capture (sale)
			if ( $zp_response[ 'status_id' ] == "101" ) {
				// Mark as paid/payment complete
				$order->update_meta_data( '_zp_charge_captured', 'yes' );
				$order->update_meta_data( '_zp_transaction_id', $transaction_id );
				$order->payment_complete( $transaction_id );
				//$order->update_status( 'completed');
			}

			// Handle Level 3
			if ( $zp_response[ 'status_id' ] == '102' || $zp_response[ 'status_id' ] == "101" ) {
				$response = $zpfn->processLevelThreeData( $order, $order_id, $transaction_id ); // Bool
				
				if ($response) { 
					$note = "";
					$note .= "Added Level 3 Data for this transaction";
					$order->add_order_note( __( $note, 'zeamster' ) );
				}
			}
			
			// Empty cart & return
			$woocommerce->cart->empty_cart();
			return $redirecturl;
		}
	}

    /**
     * Payment refund action (override of the abstract method)
     *
     * @param Int    $orderID ID of the order
     * @param Float  $amount  refund amount
     * @param String $reason  refund reason
     *
     * @return boolean
     */
    public function process_refund( $order_id, $amount = null, $reason = '' )
    {
		$order = wc_get_order( $order_id );
		
        if (!$order || !$order->get_transaction_id()) { 
            return false;
		}
		
		$zpfn = new WC_Zeamster_Functions();

		// Check for config setup 
		if ( !$zpfn->checkZpConfig() ) { 
			wc_add_notice( __( 'Gateway Error. Please Notify Store Owner', GW_ZP_ID ), 'error' );	
			return false;
		}

		// Get Transaction from API
		$response = $zpfn->getTransaction( $order->get_transaction_id() );
		
		// Check for Error Response or proceed
		if ( !is_wp_error( $response ) && $response[ 'transaction' ][ 'id' ] === $order->get_transaction_id() ) {
			
			// Check is_voidable and is_refundable to perform necessary action
			if ( $response[ 'transaction' ][ 'is_voidable' ] === true ) { 
				// Void transaction - Allow for Partial Void using Partial Reversal Endpoint
				// Get Remaining Balance after Refund (Shows how much is left AFTER refund) 
				$amountRemaining = $order->get_remaining_refund_amount();

				// Determine if a Full Void or Partial Reversal
				if ( gettype($amountRemaining) === "string" && $amountRemaining === "0.00" ) { 
					// Full Void
					$response = $zpfn->voidTransaction( $order->get_transaction_id() );
					return $zpfn->validateApiRefundResponse( $response, $order, $order_id, $amount, $expectedResponseCode = 201 );
				} elseif ( gettype($amountRemaining) === "string" && $amountRemaining !== "0.00" ) {
					// Partial Reversal
					$response = $zpfn->partialReversalTransaction( $order->get_transaction_id(), $amountRemaining );
					return $zpfn->validateApiRefundResponse( $response, $order, $order_id, $amount, $expectedResponseCode = 101 );
				}
			}
			
			// Supports Partial Refunds
			if  ( $response[ 'transaction' ][ 'is_refundable' ] === true ) { 
				$response = $zpfn->refundTransaction( $amount, $order->get_transaction_id() );
				return $zpfn->validateApiRefundResponse( $response, $order, $order_id, $amount, $expectedResponseCode = 111 );
			}
		}

		return false;
	}

	public function saveAccountCardRecord( $zp_response ) {
        if ( is_user_logged_in() && isset( $zp_response[ 'saved_account_vault_id' ] ) ) {
			$zpfn = new WC_Zeamster_Functions();
			$av_response = $zpfn->getAccountVaultRecord($zp_response[ 'saved_account_vault_id' ]);

			if ( isset ( $av_response[ "accountvault" ] ) ) { 
				$expDate = $av_response[ "accountvault" ][ "exp_date" ];

				$month = substr( $expDate, 0, 2 );
				$year = "20". substr( $expDate, 2, 2 );

				$token = new WC_Payment_Token_CC();
				$token->set_token( $zp_response[ 'saved_account_vault_id' ] );
				$token->set_gateway_id( GW_ZP_ID );
				$token->set_last4( $zp_response[ 'last_four' ] );
				$token->set_card_type( $this->getCardType( $zp_response[ 'card_type' ] ) );
				$token->set_expiry_year( $year );
				$token->set_expiry_month( $month );
				$token->set_user_id( get_current_user_id() );
				// Save the new token to the database   
				$token->save();
			}
        }
    }
	
    /**
     * Return card types
     *
     * @return Object List of card types
     */
    private function _getCardTypeOptions()
    {
        return array(
            'VISA' => 'VISA',
            'MC' => 'MasterCard',
            'AMEX' => 'American Express',
            'DISC' => 'Discover',
            'DC' => 'Diner\'s Club',
            'JCB' => 'JCB Card',
        );
	}

	private function getCardType($cardType) {
		if ('VISA' == strtoupper($cardType)) 	return "Visa";
		if ('MC' == strtoupper($cardType)) 		return "MasterCard";
		if ('AMEX' == strtoupper($cardType)) 	return "American Express";
		if ('DISC' == strtoupper($cardType)) 	return "Discover";
		if ('DC' == strtoupper($cardType)) 		return "Diners Club";
		if ('JCB' == strtoupper($cardType)) 	return "JCB Card";

		return false;
	}	
}