<?php
/**
 * Plugin Name: WooCommerce Fortis Gateway
 * Plugin URI:
 * Description: WooCommerce plugin for accepting payments through the Fortis gateway
 * Version: 1.0.8
 * Author: Fortis
 * Author URI: https://www.fortispay.com/
 * Contributors: Fortis
 * WC requires at least: 4.0.0
 * WC tested up to: 6.7.0.
 *
 * Text Domain: zp-gw
 * Domain Path: /lang/
 *
 * @author Fortis <devsupport@fortispay.com>
 */

if (!defined('ABSPATH')) {
    exit;
}

$WOOZPDIRECTNAME = 'WooCommerce Fortis Gateway';
$WOOZPDIRECTDIR = basename(__DIR__);
$WOOZPDIRECTFILE = basename(__FILE__);
$WOOZPDIRECTABSPATH = __FILE__;
$WOOZPDIRECTV = '1.0.8';

require 'includes/settings.php';
require 'includes/logger.php';

add_action('plugins_loaded', 'initWoocommerceZPGW', 0 );

/**
 * Initialize the Woocommerce plugin
 *
 * @return Void
 */
function initWoocommerceZPGW()
{
    if ( !class_exists( 'WC_Payment_Gateway_CC' ) || !class_exists( 'WC_Payment_Gateway_eCheck' ) ) {
        return;
    }

    load_plugin_textdomain( GW_ZP_PLUGIN_ID, false, dirname( plugin_basename( __FILE__ ) ) . '/lang' );

	include 'classes/class-wc-gateway-zeamster-card.php'; // Class = 'WC_Zeamster_Gateway_Card';
	include 'classes/class-wc-gw-functions.php'; // Class = WC_Zeamster_Functions
	include 'classes/class-wc-fortis-checkout-redirect.php'; // Class = WC_Fortis_Checkout_Redirect

	WC_Fortis_Checkout_Redirect::get_instance();

    /**
     * Add gateway implementations
     *
     * @param Array $methods List of available methods
     *
     * @return Array
     */
    function addZpGateway( $methods ) {
        $methods[] = 'WC_Zeamster_Gateway_Card';
        return $methods;
	}

	add_filter( 'woocommerce_payment_gateways' , 'addZpGateway' );
	add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'plugin_action_links' );

	/**
	 * Show action links on the plugin screen.
	 *
	 * @param mixed $links Plugin Action links.
	 *
	 * @return array
	 */
	function plugin_action_links( $links ) {
		$action_links = array(
			'settings' => '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=checkout&section=' . GW_ZP_ID ) . '" aria-label="' . esc_attr__( 'View Gateway Settings', GW_ZP_ID ) . '">' . esc_html__( 'Settings', GW_ZP_ID ) . '</a>',
		);

		return array_merge( $action_links, $links );
	}
	
	// Setup Ajax action hook
	add_action( 'wp_ajax_nopriv_zp_process_transaction_postback', 'zp_process_transaction_postback' );
	add_action( 'wp_ajax_zp_process_transaction_postback', 'zp_process_transaction_postback' );
}

function zp_process_transaction_postback() { 
	$options = get_option("woocommerce_". GW_ZP_ID ."_settings");

	$logger = new ZP_Logger();
	$logger->setFebug($options["debug"] === "yes" ? true : false);
	
	$order_id = $_POST[ 'order_id' ];
	$zp_response = $_POST[ 'zp_txn_response' ];
	$gw = new WC_Zeamster_Gateway_Card();

	if ( isset( $zp_response[ 'saved_account_vault_id' ] ) ) {
		if ( $zp_response[ 'saved_account_vault_id' ] != "" ) {
			$gw->saveAccountCardRecord( $zp_response );	
		}
	}
	
	$redirect_url = $gw->updateSuccessfulOrder( $order_id, $zp_response, true );
	$resp = array(
		'result'   => 'success',
		'redirect' => $redirect_url,
	);

	wp_send_json( $resp );
}

function zp_load_scripts () { 
// 	if ( !empty( $_GET ) && isset( $_GET[ 'order-pay' ] ) && isset( $_GET[ 'key' ] ) ) { 
// 		wp_enqueue_script( 'zp_ajax_payment_script',  plugin_dir_url(__FILE__) . 'javascript/process_payment.js', array('jquery') );
// 		wp_localize_script( 'zp_ajax_payment_script', 'frontendajax', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ) ) );
// 	}

    wp_enqueue_script( 'zp_ajax_payment_script',  plugin_dir_url(__FILE__) . 'javascript/process_payment.js', array('jquery') );
    wp_localize_script( 'zp_ajax_payment_script', 'frontendajax', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ) ) );
    
	wp_enqueue_script('zp_bootstrap_3', plugin_dir_url(__FILE__) . 'javascript/zp_bs_3.js', array(), false, false);
	wp_enqueue_script('zp_admin', 		plugin_dir_url(__FILE__) . 'javascript/admin.js', array(), false, false);
	wp_enqueue_style('zp_bootstrap_3_css' , plugin_dir_url(__FILE__) . 'css/zp_bs_3.css' , array(), false, 'all');

}

add_action( 'wp_enqueue_scripts', 'zp_load_scripts' );
add_filter( 'woocommerce_order_status_changed', 'zp_update_order_status', 10, 3 );
add_filter( 'woocommerce_order_button_html', 'inactive_order_button_html' );
add_filter( 'woocommerce_order_button_text', 'zeamster_place_order_button' ); 

function zp_update_order_status ( $order_id, $old_status, $new_status ) {
	$order = new WC_Order( $order_id );
	
	// Update Successful
	if ( $order->get_payment_method() == 'zeamster' && in_array( $new_status, [ 'processing' , 'completed' ], true ) ) {
		$zpfn = new WC_Zeamster_Functions();
		$captured = $order->get_meta( '_zp_charge_captured' );
		$transaction_id = $order->get_meta( '_zp_transaction_id' );

		// See if transaction has previously been captured
		if ( $captured == 'no' && in_array( $new_status, [ 'processing' , 'completed' ], true ) && $transaction_id ) {
			
			$response = $zpfn->authCompleteTransaction( $order, $order_id, $transaction_id, $order->get_total() );
			$response = $zpfn->validateApiCaptureResponse( $response, $order, $order_id, $order->get_total(), $expectedResponseCode = 101 );

			if ( !$response || $response[ "status" ] === "error") {
				$order->update_status( 'on-hold' );
				$order->add_order_note( 'UNABLE TO COMPLETE TRANSACTION.' . $response[ 'message' ] );
				$order->save();
				return false;
			}

			$complete_message = sprintf( __( 'Gateway charge complete (Charge ID: %s)', GW_ZP_ID ), $transaction_id );
			$order->add_order_note( $complete_message );
			$order->update_meta_data( '_zp_charge_captured', 'yes' );
			$order->save();
			return true;
		}

		if ( $captured == 'yes' && $transaction_id ) {
			$order->payment_complete( $transaction_id );
		}
	}
}

function zeamster_place_order_button() {
    return __( 'Pay Securely', 'woocommerce' ); 
}

function inactive_order_button_html( $button ) {
	if ( is_checkout() ) { 
		$zpfn = new WC_Zeamster_Functions();
		
		// Disable place order for Zeamster if config Invalid
		if( !$zpfn->checkZpConfig() ) {
			$style = 'style="background:Silver !important; color:white !important; cursor: not-allowed !important;"';
			$button_text = apply_filters( 'woocommerce_order_button_text', __( 'Pay Securely', 'woocommerce' ) );
			$button = '<a class="button" ' . $style . '>' . $button_text . '</a>';
		} 

		return $button;
	}
}